// Inputs
long input(0) cmd;
//array input(1) u;
//array input(2) z;
//array input(3) x;
//array input(4) f_h;
//array input(5) df_dh;
// Outputs
//ref	output(0) this;

#define ARR_U 1  // Input vector u(t)
#define ARR_Z 2  // Measurement vector z(t)
#define ARR_X 3  // Mean state (estimated) vector x
#define ARR_F 4  // f(x(t), u(t)) - state model reference, or h(x(t), u(t)) - measurement model reference
#define ARR_DF 5 // df(x(t), u(t))/dx(t) - state model Jacobian, or dh(x(t), u(t))/dx(t) - measurement model Jacobian

#define CNT_U 2 // Dimension of input vector u(t)
#define CNT_Z 2 // Dimension of measurement vector z(t)
#define CNT_X 4 // Dimension of state vector x(t)

long main(void)
{
	double value;
	//Trace(1,cmd);
	switch(cmd)
	{
	case 0:	// compute and set f(x,u) a df(x,u)/dx
		// Dimension check - can be omitted
		// Vector is a column (matrix with one column) in REXYGEN
		if(GetInArrRows(ARR_U)<CNT_U || GetInArrRows(ARR_X)<CNT_X || GetInArrRows(ARR_F)<CNT_X || GetInArrRows(ARR_DF)<CNT_X || GetInArrCols(ARR_DF)<CNT_X)
		{
			TraceWarning(11,GetInArrRows(ARR_U));
			TraceWarning(12,GetInArrRows(ARR_X));
			TraceWarning(13,GetInArrRows(ARR_F));
			TraceWarning(14,GetInArrRows(ARR_DF));
			TraceWarning(15,GetInArrCols(ARR_DF));
		}
		// Just simple linear system is used in this example
		SetInArrValue(ARR_F,0,0,GetInArrDouble(ARR_X,1,0)-GetInArrDouble(ARR_X,0,0)); // x2 - x1
		SetInArrValue(ARR_F,1,0,GetInArrDouble(ARR_X,2,0)-GetInArrDouble(ARR_X,1,0)); // x3 - x2
		SetInArrValue(ARR_F,2,0,GetInArrDouble(ARR_X,3,0)-GetInArrDouble(ARR_X,2,0)); // x4 - x3
		SetInArrValue(ARR_F,3,0,GetInArrDouble(ARR_U,0,0)-GetInArrDouble(ARR_X,3,0)); // u1 - x4
		
		SetInArrValue(ARR_DF,0,0,-1.0); // [-1 1 0 0]
		SetInArrValue(ARR_DF,0,1,1.0);
		SetInArrValue(ARR_DF,0,2,0.0);
		SetInArrValue(ARR_DF,0,3,0.0);

		SetInArrValue(ARR_DF,1,0,0.0); // [0 -1 1 0]
		SetInArrValue(ARR_DF,1,1,-1.0);
		SetInArrValue(ARR_DF,1,2,1.0);
		SetInArrValue(ARR_DF,1,3,0.0);

		SetInArrValue(ARR_DF,2,0,0.0); // [0 0 -1 1]
		SetInArrValue(ARR_DF,2,1,0.0);
		SetInArrValue(ARR_DF,2,2,-1.0);
		SetInArrValue(ARR_DF,2,3,1.0);

		SetInArrValue(ARR_DF,3,0,-0.0); // [0 0 0 -1]
		SetInArrValue(ARR_DF,3,1,-0.0);
		SetInArrValue(ARR_DF,3,2,-0.0);
		SetInArrValue(ARR_DF,3,3,-1.0);
		break;
	case 1:	// compute and set h1(x) a dh1(x)/dx; e.g. 1st set of outputs measurement (for input EKF:nz = 1)
		// outputs just lineary depend on state variables in this example
		value = 2.0*GetInArrDouble(ARR_X,1,0) + 0.5*GetInArrDouble(ARR_X,3,0);
		SetInArrValue(ARR_F,0,0,GetInArrDouble(ARR_X,0,0)); // x1
		SetInArrValue(ARR_F,1,0,value); // 2*x2 + 0.5*x4

		SetInArrValue(ARR_DF,0,0,1.0);
		SetInArrValue(ARR_DF,0,1,0.0);
		SetInArrValue(ARR_DF,0,2,0.0);
		SetInArrValue(ARR_DF,0,3,0.0);

		SetInArrValue(ARR_DF,1,0,0.0);
		SetInArrValue(ARR_DF,1,1,2.0);
		SetInArrValue(ARR_DF,1,2,0.0);
		SetInArrValue(ARR_DF,1,3,0.5);
		break;
	case 2:	//compute and set h2(x) a dh2(x)/dx; e.g. 2nd set of outputs measurement (for input EKF:nz = 2)
		//not used in this example
		break;
	default:
		TraceError(2,cmd);
		return -1;
	}
	// The vector z is not needed for the linear case. However, it needs to be used in REXLANG, otherwise the
	// Error 68: Validation error -218 (Input connection error) in 'ekf_task.REXLANG:u2' will pop up during compilation.
	// (because the input arrays have an implicit declaration)
	TraceVerbose(20,GetInArrDouble(ARR_Z,0,0));
	return 0;
}
